<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';

require_once 'bilink_channel.php';

///////////////////////////////////////////////////////////////////////////

class BilinkTv extends AbstractTv
{
    public function __construct()
    {
        parent::__construct(
            AbstractTv::MODE_CHANNELS_N_TO_M,
            BilinkConfig::TV_FAVORITES_SUPPORTED,
            false);
    }

    public function get_fav_icon_url()
    {
        return BilinkConfig::FAV_CHANNEL_GROUP_ICON_PATH;
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    protected function load_channels(&$plugin_cookies)
    {
        $query = '&query=get_channels';
   
        $xml = HD::load_xml($plugin_cookies, $query, 'Невозможно получить список каналов!');

        if ($xml->getName() !== 'entity')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'tv_info'");
            throw new Exception('Invalid XML document');
        }

        $this->channels = new HashedArray();
        $this->groups = new HashedArray();

        if ($this->is_favorites_supported())
        {
            $this->groups->put(
                new FavoritesGroup(
                    $this,
                    '__favorites',
                    BilinkConfig::FAV_CHANNEL_GROUP_CAPTION,
                    BilinkConfig::FAV_CHANNEL_GROUP_ICON_PATH));
        }

        $this->groups->put(
            new AllChannelsGroup(
                $this,
                BilinkConfig::ALL_CHANNEL_GROUP_CAPTION,
                BilinkConfig::ALL_CHANNEL_GROUP_ICON_PATH));
	
	
	$ch_arr=$xml->row[0]->xpath("array[@name='channels']");

        foreach ($ch_arr[0] as $xml_tv_channel)
        {
            if ($xml_tv_channel->getName() !== 'row')
            {
                hd_print("Error: unexpected node '" . $xml_tv_channel->getName() .
                    "'. Expected: 'tv_channel'");
                throw new Exception('Invalid XML document');
            }

	    $number = $xml_tv_channel->xpath("item[@name='number']");
	    $title = $xml_tv_channel->xpath("item[@name='title']");
	    $icon = $xml_tv_channel->xpath("item[@name='icon_45_45']");
	    $has_archive = $xml_tv_channel->xpath("item[@name='has_record']");
	    $is_protected = $xml_tv_channel->xpath("item[@name='has_passwd']");

	    if ( intval($is_protected[0]) == 0 )
		$flag = false;
	    else
		$flag = true;

	    if ($flag == false || $plugin_cookies->show_pc != 'no')
	    {
		    $channel =
		        new BilinkChannel(
		            strval($number[0]),
		            strval($title[0]),
		            strval($icon[0]),
		            intval($number[0]),
		            intval(6),
		            intval(6),
			    intval($has_archive[0]),
			    $flag);

            $this->channels->put($channel);
	    }

        }

	$categ_arr=$xml->row[0]->xpath("array[@name='categories']");

        foreach ($categ_arr[0] as $xml_tv_category)
        {
            if ($xml_tv_category->getName() !== 'row')
            {
                hd_print("Error: unexpected node '" . $xml_tv_category->getName() .
                    "'. Expected: 'tv_category'");
                throw new Exception('Invalid XML document');
            }
	    $number = $xml_tv_category->xpath("item[@name='number']");
	    $title = $xml_tv_category->xpath("item[@name='title']");
	 
	    $group = new DefaultGroup(
                    strval($number[0]),
                    strval($title[0]),
                    strval($this->path[strval($number[0])]));
            $this->groups->put($group);

	    $ch_arr = $xml_tv_category->xpath("array[@name='channels']");
	    foreach($ch_arr[0] as $ch_num)
	    {
		$channel = $this->channels->get(intval($ch_num));
		if ($channel)
		{
		    // Link group and channel.
		    $channel->add_group($group);
                    $group->add_channel($channel);
		}
	    }
        }

        if (BilinkConfig::CHANNEL_SORT_FUNC_CB != null)
            $this->channels->usort(BilinkConfig::CHANNEL_SORT_FUNC_CB);
    }

    ///////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies)
    {
        $query = '&query=get_epg&key=number|start|period&value='.$channel_id.'|'.$day_start_ts.'|86400';
	try{
            $xml = HD::load_xml($plugin_cookies, $query, 'Невозможно получить телегид!');
	}
	catch(Exception $e)
	{
	    hd_print('Ошибка: '.$e->getMessage());
	    return array();
	}

        if ($xml->getName() !== 'entity')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'NewDataSet'");
            throw new Exception('Invalid XML document');
        }
	
	$epg = array();
	$message=$xml->row[0]->xpath("item[@name='message']");
	if (strcmp($xml['status'], 'success') != 0)
	    if (strcmp($xml['status'], 'error') == 0 && strcmp($message[0], 'No data') == 0)
		return
		    new EpgIterator($epg);
	   else
	    throw new DuneException(
                'HTTP network error', 2,
                ActionFactory::show_error(
                    true,
                    'Ошибка доступа!',
                    array(
                        'Ошибка доступа!')));

        
	$ch_num = $xml->row[0]->xpath("item[@name='number']");
	if (intval($ch_num[0]) != intval($channel_id))
	     throw new DuneException(
                'HTTP network error', 3,
                ActionFactory::show_error(
                    true,
                    'Ошибка номера канала!',
                    array('Ошибка номера канала!')));

	$pr_arr = $xml->row[0]->xpath("array[@name='programmes']");

        foreach ($pr_arr[0] as $epg_item)
        {
            if ($epg_item->getName() !== 'row')
            {
                hd_print("Error: unexpected node '" . $epg_item->getName() .
                    "'. Expected: 'epg_item'");
                throw new Exception('Invalid XML document');
            }
	    
	    $title = $epg_item->xpath("item[@name='title']");
	    $start = $epg_item->xpath("item[@name='ut_start']");
	    $stop = $epg_item->xpath("item[@name='ut_stop']");
	    $desc = $epg_item->xpath("item[@name='desc']");
	   
            $epg[] =
                new DefaultEpgItem(
                    strval($title[0]),
                    strval($desc[0]),
                    intval($start[0]),
                    intval($stop[0]));
        }

        return
            new EpgIterator($epg);
    }


    
}

///////////////////////////////////////////////////////////////////////////
?>
