<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/tv/channel.php';

///////////////////////////////////////////////////////////////////////////

class DefaultChannel implements Channel
{
    protected $_id;
    protected $_title;
    protected $_icon_url;

    protected $_groups; // Array<Group>

    public function __construct($id, $title, $icon_url)
    {
        $this->_id = $id;
        $this->_title = $title;
        $this->_icon_url = $icon_url;

        $this->_groups = array();
    }

    public function get_id()
    { return $this->_id; }

    public function get_title()
    { return $this->_title; }

    public function get_icon_url()
    { return $this->_icon_url; }

    public function get_groups()
    { return $this->_groups; }

    public function get_number()
    { return -1; }

    public function has_archive()
    { return true; }

    public function is_protected($plugin_cookies)
    { return false; }

    public function get_past_epg_days()
    { return 14; }

    public function get_future_epg_days()
    { return 7; }

    public function get_archive_past_sec()
    { return 14 * 86400; }

    public function get_archive_delay_sec()
    { 
	return 0; 
    }

    public function get_buffering_ms()
    { return 0; }

    public function get_timeshift_hours()
    { return 0; }

    public function get_streaming_url(&$plugin_cookies, $archive_ts, $protect_code)
    {
	if ($this->real_is_protected())
	    $passwd = md5($plugin_cookies->token.md5($protect_code));
	if ($archive_ts != -1)
	{
	    $query = '&query=get_epg&key=number|start|count&value='.$this->get_id().'|'.$archive_ts.'|2';
	    $xml = HD::load_xml($plugin_cookies, $query, 'Невозможно получить телепередачу!');
	    if ($xml->getName() !== 'entity')
            {
                hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'NewDataSet'");
                throw new Exception('Invalid XML document');
            } 
	    $pr_arr = $xml->row[0]->xpath("array[@name='programmes']");
	    foreach ($pr_arr[0] as $epg_item)
            {
                if ($epg_item->getName() !== 'row')
                {
                    hd_print("Error: unexpected node '" . $epg_item->getName() .
                        "'. Expected: 'epg_item'");
                    throw new Exception('Invalid XML document');
                }
		$start = $epg_item->xpath("item[@name='ut_start']");
		if (intval($start[0]) == $archive_ts)
		{
		    $pid = $epg_item->xpath("item[@name='pid']");
		    if ($this->real_is_protected())
			$query = '&query=get_url&key=pid|passwd&value='.strval($pid[0]).'|'.$passwd;
		    else
		    	$query = '&query=get_url&key=pid&value='.strval($pid[0]);
		}
	    }
	    if (!isset($pid))
	    {
		//throw new Exception('Invalid epg from middleware!');
		if ($this->real_is_protected())
            	    $query = '&query=get_url&key=number|passwd&value='.$this->get_id().'|'.$passwd;
	        else
		    $query = '&query=get_url&key=number&value='.$this->get_id();
	        $xml = HD::load_xml($plugin_cookies, $query, 'Невозможно получить поток!');
	    }
	    else
	    {
		try{
		    $xml = HD::load_xml($plugin_cookies, $query, 'Архив недоступен!');
		}
		catch(Exception $e)
		{
		    throw new DuneException(
			'Load XML error', 5,
			ActionFactory::show_error(
			    true,
			    'Архив недоступен!',
			    array()));
		}
	    }
	}
	else
	{
	    if ($this->real_is_protected())
            	$query = '&query=get_url&key=number|passwd&value='.$this->get_id().'|'.$passwd;
	    else
		$query = '&query=get_url&key=number&value='.$this->get_id();
	    $xml = HD::load_xml($plugin_cookies, $query, 'Невозможно получить поток!');
	}
    
	        
	if($xml == 'incorrect parent code')
	    return 'incorrect parent code';

        if ($xml->getName() !== 'entity')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'NewDataSet'");
            throw new Exception('Invalid XML document');
        } 
	$url = $xml->row[0]->xpath("item[@name='url']");

	return strval($url[0]); 
    }

    ///////////////////////////////////////////////////////////////////////

    public function add_group($group)
    { $this->_groups[] = $group; }
}

///////////////////////////////////////////////////////////////////////////
?>
