<?php
///////////////////////////////////////////////////////////////////////////

class HD
{
    public static function is_map($a)
    {
        return is_array($a) &&
            array_diff_key($a, array_keys(array_keys($a)));
    }

    ///////////////////////////////////////////////////////////////////////

    public static function has_attribute($obj, $n)
    {
        $arr = (array) $obj;
        return isset($arr[$n]);
    }
    ///////////////////////////////////////////////////////////////////////

    public static function get_map_element($map, $key)
    {
        return isset($map[$key]) ? $map[$key] : null;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function starts_with($str, $pattern)
    {
        return strpos($str, $pattern) === 0;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function format_timestamp($ts, $fmt = null)
    {
        // NOTE: for some reason, explicit timezone is required for PHP
        // on Dune (no builtin timezone info?).

        if (is_null($fmt))
            $fmt = 'Y:m:d H:i:s';

        $dt = new DateTime('@' . $ts);
        return $dt->format($fmt);
    }

    ///////////////////////////////////////////////////////////////////////

    public static function format_duration($msecs)
    {
        $n = intval($msecs);

        if (strlen($msecs) <= 0 || $n <= 0)
            return "--:--";

        $n = $n / 1000;
        $hours = $n / 3600;
        $remainder = $n % 3600;
        $minutes = $remainder / 60;
        $seconds = $remainder % 60;

        if (intval($hours) > 0)
        {
            return sprintf("%d:%02d:%02d", $hours, $minutes, $seconds);
        }
        else
        {
            return sprintf("%02d:%02d", $minutes, $seconds);
        }
    }

    ///////////////////////////////////////////////////////////////////////

    public static function encode_user_data($a, $b = null)
    {
        $media_url = null;
        $user_data = null;

        if (is_array($a) && is_null($b))
        {
            $media_url = '';
            $user_data = $a;
        }
        else
        {
            $media_url = $a;
            $user_data = $b;
        }

        if (!is_null($user_data))
            $media_url .= '||' . json_encode($user_data);

        return $media_url;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function decode_user_data($media_url_str, &$media_url, &$user_data)
    {
        $idx = strpos($media_url_str, '||');

        if ($idx === false)
        {
            $media_url = $media_url_str;
            $user_data = null;
            return;
        }

        $media_url = substr($media_url_str, 0, $idx);
        $user_data = json_decode(substr($media_url_str, $idx + 2));
    }

    ///////////////////////////////////////////////////////////////////////

    public static function create_regular_folder_range($items,
        $from_ndx = 0, $total = -1, $more_items_available = false)
    {
        if ($total === -1)
            $total = $from_ndx + count($items);

        if ($from_ndx >= $total)
        {
            $from_ndx = $total;
            $items = array();
        }
        else if ($from_ndx + count($items) > $total)
        {
            array_splice($items, $total - $from_ndx);
        }

        return array
        (
            PluginRegularFolderRange::total => intval($total),
            PluginRegularFolderRange::more_items_available => $more_items_available,
            PluginRegularFolderRange::from_ndx => intval($from_ndx),
            PluginRegularFolderRange::count => count($items),
            PluginRegularFolderRange::items => $items
        );
    }

    ///////////////////////////////////////////////////////////////////////

    public static function http_get_document($url, $opts = null)
    {
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT,    15);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,    true);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($ch, CURLOPT_TIMEOUT,           15);
        curl_setopt($ch, CURLOPT_USERAGENT,         'DuneHD/1.0');
        curl_setopt($ch, CURLOPT_URL,               $url);

        if (isset($opts))
        {
            foreach ($opts as $k => $v)
                curl_setopt($ch, $k, $v);
        }

        hd_print("HTTP fetching '$url'...");

        $content = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if($content === false)
        {
            $err_msg = "HTTP error: $http_code (" . curl_error($ch) . ')';
            hd_print($err_msg);
            throw new Exception($err_msg);
        }

        if ($http_code != 200)
        {
            $err_msg = "HTTP request failed ($http_code)";
            hd_print($err_msg);
            throw new Exception($err_msg);
        }

        hd_print("HTTP OK ($http_code)");

        curl_close($ch);

        return $content;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function http_post_document($url, $post_data)
    {
        return self::http_get_document($url,
            array
            (
                CURLOPT_POST => true,
                CURLOPT_POSTFIELDS => $post_data
            ));
    }

    ///////////////////////////////////////////////////////////////////////

    public static function parse_xml_document($doc)
    {
        $xml = simplexml_load_string($doc);

        if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        return $xml;
    }

    ///////////////////////////////////////////////////////////////////////

    public static function make_json_rpc_request($op_name, $params)
    {
        static $request_id = 0;

        $request = array
        (
            'jsonrpc' => '2.0',
            'id' => ++$request_id,
            'method' => $op_name,
            'params' => $params
        );

        return $request;
    }

    ///////////////////////////////////////////////////////////////////////////

    public static function get_mac_addr()
    {
        static $mac_addr = null;

        if (is_null($mac_addr))
        {
            $mac_addr = shell_exec(
                'ifconfig  eth0 | head -1 | sed "s/^.*HWaddr //"');

            $mac_addr = trim($mac_addr);

            hd_print("MAC Address: '$mac_addr'");
        }

        return $mac_addr;
    }

    ///////////////////////////////////////////////////////////////////////////

    // TODO: localization
    private static $MONTHS = array(
        'January',
        'February',
        'March',
        'April',
        'May',
        'June',
        'July',
        'August',
        'September',
        'October',
        'November',
        'December',
    );

    public static function format_date_time_date($tm)
    {
        $lt = localtime($tm);
        $mon = self::$MONTHS[$lt[4]];
        return sprintf("%02d %s %04d", $lt[3], $mon, $lt[5] + 1900);
    }

    public static function format_date_time_time($tm, $with_sec = false)
    {
        $format = '%H:%M';
        if ($with_sec)
            $format .= ':%S';
        return strftime($format, $tm);
    }

    public static function print_backtrace()
    {
        hd_print('Back trace:');
        foreach (debug_backtrace() as $f)
        {
            hd_print(
                '  - ' . $f['function'] . 
                ' at ' . $f['file'] . ':' . $f['line']);
        }
    }
 ///////////////////////////////////////////////////////////////////////////

    public static function authorization(&$plugin_cookies)
    {
	$login= $plugin_cookies->login;
	$pass= $plugin_cookies->pass;

	//=====================этап 1====================//
	$doc = HD::http_get_document('http://ott.bilink.ua/auth.xml');
	$xml = simplexml_load_string($doc);
	if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'entity')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'tv_info'");
            throw new Exception('Invalid XML document');
        }
	$rand = $xml->row->xpath("item[@name='rand']");
	$sid = $xml->row->xpath("item[@name='sid']");
	$send_pass=md5($rand[0].md5($pass));
	
	//=====================этап 2====================//
	$doc = HD::http_get_document(sprintf('http://ott.bilink.ua/auth.xml?device=%s&version=%s&sid=%s&login=%s&passwd=%s&serial=%s','dune_hd','1.0.0',$sid[0],$login,$send_pass,HD::get_mac_addr()));
	$xml = simplexml_load_string($doc);
	if ($xml === false)
        {
            hd_print("Error: can not parse XML document.");
            hd_print("XML-text: $doc.");
            throw new Exception('Illegal XML document');
        }

        if ($xml->getName() !== 'entity')
        {
            hd_print("Error: unexpected node '" . $xml->getName() . "'. Expected: 'tv_info'");
            throw new Exception('Invalid XML document');
        }
	
	if (strcmp($xml['status'], 'success') != 0)
	    throw new DuneException(
                'Authorization error', 1,
                ActionFactory::show_error(
                    true,
                    'Ошибка доступа',
                    array(
                        'Не удаётся подключиться к IPTV-серверу.',
                        'Проверьте введенные логин и пароль.')));
	    
	$portal = $xml->row->xpath("item[@name='portal']");
	$token = $xml->row->xpath("item[@name='token']");
	//$ttl = $xml->row->xpath("item[@name='ttl']");
	$plugin_cookies->portal= strval($portal[0]);
	$plugin_cookies->token= strval($token[0]);
    }

    ///////////////////////////////////////////////////////////////////////////
    
    public static function load_xml(&$plugin_cookies, $query_end, $error_message)
    {
	$query = $plugin_cookies->portal.'?token='.$plugin_cookies->token.$query_end;

	try
	{
	    $doc = HD::http_get_document($query);
	}
	catch (Exception $e)
	{
	    throw new DuneException(
	        'HTTP network error', 0,
	        ActionFactory::show_error(
	            true,
	            'Ошибка связи',
	            array(
	                'Не удаётся подключиться к IPTV-серверу.',
	                'Проверьте подключение к сети Интернет.')));
	}
	
	hd_print("XML is: ".$doc);
	$xml = simplexml_load_string($doc);

	if ($xml === false)
	{
	    hd_print("Error: can not parse XML document.");
	    hd_print("XML-text: $doc.");
	    throw new Exception('Illegal XML document');
	}
	
	if (strcmp($xml['status'],'success') != 0)
	{
	    $code = $xml->row[0]->xpath("item[@name='code']");
	    if (strval($code[0]) == '4002')
	    {
		
		HD::authorization($plugin_cookies);
		
                $query = $plugin_cookies->portal.'?token='.$plugin_cookies->token.$query_end;

		try
		{
		    $doc = HD::http_get_document($query);
		}
		catch (Exception $e)
		{
		    throw new DuneException(
			'HTTP network error', 0,
			ActionFactory::show_error(
			    true,
			    'Ошибка связи',
			    array(
			        'Не удаётся подключиться к IPTV-серверу.',
			        'Проверьте подключение к сети Интернет.')));
		}
		
		$xml = simplexml_load_string($doc);

		if ($xml === false)
		{
		    hd_print("Error: can not parse XML document.");
		    hd_print("XML-text: $doc.");
		    throw new Exception('Illegal XML document');
		}
	
		if (strcmp($xml['status'],'success') != 0)
		{
		    if (strval($code[0]) == '4008')
		    {
			return 'incorrect parent code';
		    }
		    else
		    //throw new Exception($error_message.' - after reauthorization');
		        throw new DuneException(
			    'Load XML error', 5,
			    ActionFactory::show_error(
			        true,
			        'Ошибка получения данных',
			        array($error_message.' - after reauthorization')));
		}
	    }
	    else if (strval($code[0]) == '4008')
	    {
		return 'incorrect parent code';
	    }
	    else if (strval($code[0]) == '4006')
		//throw new Exception($error_message);
		throw new DuneException(
			'Other user error', 7,
			ActionFactory::show_error(
			    true,
			    'Ваша учетная запись уже используется!',
			    array('Кто-то другой использует эту учетную запись...')));
	    else
		//throw new Exception($error_message);
		throw new DuneException(
			'Load XML error', 5,
			ActionFactory::show_error(
			    true,
			    'Ошибка получения данных',
			    array($error_message)));
	}
	return $xml;
    }

    
}

///////////////////////////////////////////////////////////////////////////
?>
